import * as actions from '../src/actions';
import { assert } from 'chai';
var actionTypes = actions.actionTypes;
describe('action creators', function () {
    ['start', 'stop'].forEach(function (actionKey) {
        describe(actionKey + "()", function () {
            it('should accept a string action', function () {
                var action = actions[actionKey]('test');
                assert.equal(action.type, actionTypes[actionKey]);
                assert.deepEqual(action, {
                    type: actionTypes[actionKey],
                    exec: undefined,
                    activity: {
                        type: 'test',
                        exec: undefined,
                        id: 'test'
                    }
                });
            });
            it('should accept an action object', function () {
                var action = actions[actionKey]({ type: 'test', foo: 'bar' });
                assert.equal(action.type, actionTypes[actionKey]);
                assert.deepEqual(action, {
                    type: actionTypes[actionKey],
                    exec: undefined,
                    activity: {
                        type: 'test',
                        id: undefined,
                        foo: 'bar'
                    }
                });
            });
            it('should accept an activity definition', function () {
                var action = actions[actionKey]({
                    type: 'test',
                    foo: 'bar',
                    src: 'someSrc'
                });
                assert.equal(action.type, actionTypes[actionKey]);
                assert.deepEqual(action, {
                    type: actionTypes[actionKey],
                    exec: undefined,
                    activity: {
                        type: 'test',
                        id: undefined,
                        foo: 'bar',
                        src: 'someSrc'
                    }
                });
            });
        });
    });
    describe('send()', function () {
        it('should accept a string event', function () {
            var action = actions.send('foo');
            assert.deepEqual(action, {
                to: undefined,
                type: actionTypes.send,
                event: { type: 'foo' },
                delay: undefined,
                id: 'foo'
            });
        });
        it('should accept an event object', function () {
            var action = actions.send({ type: 'foo', bar: 'baz' });
            assert.deepEqual(action, {
                to: undefined,
                type: actionTypes.send,
                event: { type: 'foo', bar: 'baz' },
                delay: undefined,
                id: 'foo'
            });
        });
        it('should accept an id option', function () {
            var action = actions.send('foo', { id: 'foo-id' });
            assert.deepEqual(action, {
                to: undefined,
                type: actionTypes.send,
                event: { type: 'foo' },
                delay: undefined,
                id: 'foo-id'
            });
        });
        it('should accept a delay option', function () {
            var action = actions.send('foo', { delay: 1000 });
            assert.deepEqual(action, {
                to: undefined,
                type: actionTypes.send,
                event: { type: 'foo' },
                delay: 1000,
                id: 'foo'
            });
        });
        it('should accept a delay option (expression)', function () {
            var action = actions.send('RECEIVED', {
                delay: function (ctx, e) {
                    return ctx.delay + ('value' in e ? e.value : 0);
                }
            });
            var resolvedAction = actions.resolveSend(action, { delay: 100 }, { type: 'EVENT', value: 50 });
            assert.equal(resolvedAction.delay, 150);
        });
    });
});
//# sourceMappingURL=actionCreators.test.js.map